import {
    LimeObject,
    LimeProperty,
    LimeWebComponent,
    LimeWebComponentContext,
    LimeWebComponentPlatform,
    PlatformServiceName,
    SelectLimeObjects,
} from '@limetech/lime-web-components';
import { Component, Event, EventEmitter, h, Prop, State } from '@stencil/core';
import {
    ArticleSelectorComponent,
    CpqArticle,
    CpqDeal,
    CpqItem,
    CpqOrder,
} from 'limepkg-cpq-order-lwc-components';

const coffeArticleGroupId = 1004;
const discountA = 30;
const discountB = 12.5;
const discountC = 5;

@Component({
    tag: 'example-article-selector-company',
    shadow: true,
})
export class ArticleSelectorCompany
    implements LimeWebComponent, ArticleSelectorComponent
{
    @Prop()
    public platform: LimeWebComponentPlatform;

    @Prop()
    public context: LimeWebComponentContext;

    @Prop()
    public articleGroupProperty: LimeProperty;

    @Prop()
    public articleProperty: LimeProperty;

    @Prop()
    public item: CpqItem;

    @Prop()
    public itemParent: CpqDeal | CpqOrder;

    @Event()
    itemChange: EventEmitter<CpqItem>;

    @State()
    @SelectLimeObjects({
        limetype: 'company',
        map: [getCompanyFromItemParent],
    })
    private companyFromParent?: LimeObject;

    @SelectLimeObjects({
        limetype: 'deal',
        map: [getCompanyFromDeal],
    })
    private companyFromDeal?: LimeObject;

    private get company(): LimeObject {
        return this.companyFromParent || this.companyFromDeal;
    }

    private get discountPercent(): number {
        switch (this.company?.classification) {
            case 'A':
                return discountA;
            case 'B':
                return discountB;
            case 'C':
                return discountC;
            default:
                return 0;
        }
    }

    private get classificationText(): string {
        return this.company
            ?.getLimetype()
            .getProperty('classification')
            .options.find(option => option.key === this.company?.classification)
            ?.text;
    }

    public componentWillLoad() {
        if (this.companyFromParent?.classification) {
            return;
        }

        const limeObjectRepository = this.platform.get(
            PlatformServiceName.LimeObjectRepository
        );
        const companyId = this.itemParent.getProperty('company');

        if (typeof companyId === 'number') {
            limeObjectRepository.loadObject('company', companyId);
        }
    }

    public render() {
        return [
            <div>{this.company?.name}</div>,
            this.renderCustomerDiscountInformation(),
            <coc-article-selector
                platform={this.platform}
                context={this.context}
                articleGroupProperty={this.articleGroupProperty}
                articleProperty={this.articleProperty}
                item={this.item}
                onArticleChange={this.handleArticleChange}
            />,
        ];
    }

    public renderCustomerDiscountInformation() {
        if (this.discountPercent > 0) {
            return [
                <div>
                    <i>
                        {this.classificationText}: {this.discountPercent}%
                        discount on everything but coffee
                    </i>
                </div>,
            ];
        }
    }

    private handleArticleChange = (event: CustomEvent<CpqArticle | null>) => {
        const article = event.detail;
        if (!article) {
            return;
        }

        event.stopPropagation();

        let discountPercent: number;

        if (article.articleGroup?.id === coffeArticleGroupId) {
            discountPercent = 0;
        } else {
            discountPercent = this.discountPercent;
        }

        const item: CpqItem = {
            ...this.item,
            article: article,
            description: article?.description,
            pricePerUnit: article?.price,
            vat: article?.vat,
            costPrice: article?.costPrice,
            unit: article?.unit,
            discountClass: 'percent',
            discountPercent: discountPercent,
        };

        this.itemChange.emit(item);
    };
}

function getCompanyFromItemParent(
    this: ArticleSelectorCompany,
    companies: LimeObject[]
) {
    const companyId = this.itemParent.getProperty('company');

    if (typeof companyId === 'number') {
        return companies.find(company => company.id === companyId);
    }
}

function getCompanyFromDeal(this: ArticleSelectorCompany, deals: LimeObject[]) {
    let parentContext = this.context.parent;

    if (parentContext?.limetype === 'order') {
        parentContext = parentContext.parent;
    }

    if (parentContext?.limetype === 'deal') {
        return deals
            .find(deal => deal.id === parentContext.id)
            ?.getValue('company');
    }
}
